@@ -1,29 +1,14 @@
 /*
+ * oFono - Open Source Telephony
+ * Copyright (C) 2008-2011  Intel Corporation
  *
- *  oFono - Open Source Telephony
- *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#define _GNU_SOURCE
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -38,8 +23,6 @@
 #include "gatchat.h"
 #include "gatresult.h"
 
-#include "huaweimodem.h"
-
 static const char *cvoice_prefix[] = { "^CVOICE:", NULL };
 
 struct audio_settings_data {
@@ -113,18 +96,9 @@ static void huawei_audio_settings_remove(struct ofono_audio_settings *as)
 	g_free(asd);
 }
 
-static struct ofono_audio_settings_driver driver = {
-	.name		= "huaweimodem",
+static const struct ofono_audio_settings_driver driver = {
 	.probe		= huawei_audio_settings_probe,
 	.remove		= huawei_audio_settings_remove,
 };
 
-void huawei_audio_settings_init(void)
-{
-	ofono_audio_settings_driver_register(&driver);
-}
-
-void huawei_audio_settings_exit(void)
-{
-	ofono_audio_settings_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(audio_settings, huaweimodem, &driver)
